from gp2x.locals import *
import gp2x

class GuiElement:
	"""
	A single GUI element such as a numberbox or label.
	"""
	def __init__(self, x, y):
		self.x = self.AlignX(x, self.width)
		self.y = self.AlignY(y, self.height)
		# whether i am selectable
		self.selectable = True
		# whether i am currently selected or not
		self.selected = False
		# list of things that are next to me, ordered by button
		self.adjacent = []
		for i in range(8): self.adjacent.append(False)
		
		# name of the send variable
		self.send = ""
		self.receive = ""
	
	def Draw(self):
		"""
		You can override this if the thing can't be selected.
		"""
		if self.selected:
			self.DrawSelected()
		else:
			self.DrawNormal()
		
	def AlignX(self, amount, width):
		"""
		Align from the right if a negative number is entered.
		"""
		if amount > 0:
			return amount
		elif amount < 0:
			return SCREEN_WIDTH - width + amount

	def AlignY(self, amount, width):
		"""
		Align from the bottom if a negative number is entered.
		"""
		if amount > 0:
			return amount
		elif amount < 0:
			return SCREEN_HEIGHT - width + amount
	
	def DrawSelected(self):
		"""
		Override this to draw the gui element in selected mode.
		"""
	
	def DrawNormal(self):
		"""
		Override this to draw the gui element when unselected.
		"""
	
	def Buttons(self, changed, held):
		"""
		Override this to handle button evens on a gui element.
		"""
	
