import pygame
from pygame.locals import *
from gp2x.locals import *
import gp2x
from gui.Label import Label
from math import sqrt, pow
from PdParser import PdParser
from sys import argv

class Page(PdParser):
	def __init__(self, parent):
		self.gfx = gp2x.gfx
		self.parent = parent
		self.selectedElement = None
		self.elements = []
		PdParser.__init__(self, argv[1])
		self.Parse()
		self.GenerateAdjacent()
	
	def Draw(self):
		"""
		Use SDL calls to draw me onto the pygame screen
		"""
		for e in self.elements:
			e.Draw()
	
	def AddElement(self, element):
		"""
		Add a GUI element to the list of elements contained by this page.
		"""
		# if we haven't selected a gui element yet and this is one then select it
		if not self.selectedElement and not type(element) == Label:
			self.Select(element)
		
		# append this element to our list of elements
		self.elements.append(element)
	
	def GenerateAdjacent(self):
		# compare every element to every other element
		for e1 in self.elements:
			if e1.selectable:
				# figure out what is next to us on which side and see if it's closest
				# (for joystick navigation)
				for e in self.elements:
					if e != e1 and e.selectable:
						# which side of each element is the new element on?
						side = self.SideOf(e1, e)
						# if this new guy is closer then use him as our new right
						if not e.adjacent[side] or self.Distance(e, e1) < self.Distance(e, e.adjacent[side]):
							e.adjacent[side] = e1
	
	def SideOf(self, orig, element):
		"""
		Which side of the gui element orig is the element "element" on?
		Figures this out using a graph of quadrants that look like:
		f(x) = x
		and
		f(x) = -x
		treating orig's coordinates as the origin
		"""
		nx = element.x - orig.x
		ny = element.y - orig.y
		
		# test for bottom/right quadrant
		if nx > ny:
			if ny > -nx:
				return BUTTON_LEFT
			else:
				return BUTTON_DOWN
		# top/left quadrant
		else:
			if ny > -nx:
				return BUTTON_UP
			else:
				return BUTTON_RIGHT
	
	def Select(self, which):
		"""
		Select an element unselecting the old one
		"""
		if (self.selectedElement):
			self.selectedElement.selected = False
		self.selectedElement = which
		self.selectedElement.selected = True
	
	def Update(self):
		"""
		Do anything else I am supposed to do
		"""
	
	def Buttons(self):
		"""
		See if I care about any of the buttons that were pressed on the joypad
		"""
		changed = self.parent.changed
		held = self.parent.held

		# if the action button is held (B) then they pass the controls into the selected object
		if held[BUTTON_B]:
			self.selectedElement.Buttons(changed, held)
		# otherwise select the next door element
		else:
			for b in [BUTTON_LEFT, BUTTON_RIGHT, BUTTON_DOWN, BUTTON_UP]:
				if changed[b] and held[b] and self.selectedElement.adjacent[b]:
					self.Select(self.selectedElement.adjacent[b])
	
	def Distance(self, first, second):
		"""
		Compute the distance between two GUI elements.
		"""
		return sqrt(pow(second.x - first.x, 2) + pow(second.y - first.y, 2))
